@@ -3,25 +3,27 @@
 #' for a given forecasting horizon by estimating the best linear predictors
 #' @param object \code{fnets} object
 #' @param x input time series matrix, with each row representing a variable
-#' @param h forecasting horizon
+#' @param n.ahead forecasting horizon
 #' @param fc.restricted whether to forecast using a restricted or unrestricted, blockwise VAR representation of the common component
 #' @param r number of restricted factors, or a string specifying the factor number selection method when \code{fc.restricted = TRUE};
 #'  possible values are:
 #' \itemize{
-#'    \item{\code{"ic"}}{ information criteria of Bai and Ng (2002)}
-#'    \item{\code{"er"}}{ eigenvalue ratio}
+#'    \item{\code{"ic"}}{ information criteria of Alessi, Barigozzi & Capasso (2010))}
+#'    \item{\code{"er"}}{ eigenvalue ratio of Ahn & Horenstein (2013)}
 #' }
 #' @return a list containing
 #' \item{is}{ in-sample estimator of the common component}
 #' \item{fc}{ forecasts of the common component for a given forecasting horizon \code{h}}
 #' \item{r}{ restricted factor number}
-#' \item{h}{ forecast horizon}
+#' \item{n.ahead}{ forecast horizon}
 #' @references Ahn, S. C. & Horenstein, A. R. (2013) Eigenvalue ratio test for the number of factors. Econometrica, 81(3), 1203--1227.
+#' @references Alessi, L., Barigozzi, M., and Capasso, M. (2010) Improved penalization for determining the number of factors in approximate factor models. Statistics & Probability Letters, 80(23-24):1806–1813.
 #' @references Barigozzi, M., Cho, H. & Owens, D. (2022) FNETS: Factor-adjusted network estimation and forecasting for high-dimensional time series. arXiv preprint arXiv:2201.06110.
-#' @references Forni, M., Hallin, M., Lippi, M. & Reichlin, L. (2005). The generalized dynamic factor model: one-sided estimation and forecasting. Journal of the American Statistical Association, 100(471), 830--840.
-#' @references Forni, M., Hallin, M., Lippi, M. & Zaffaroni, P. (2017). Dynamic factor models with infinite-dimensional factor space: Asymptotic analysis. Journal of Econometrics, 199(1), 74--92.
+#' @references Forni, M., Hallin, M., Lippi, M. & Reichlin, L. (2005) The generalized dynamic factor model: one-sided estimation and forecasting. Journal of the American Statistical Association, 100(471), 830--840.
+#' @references Forni, M., Hallin, M., Lippi, M. & Zaffaroni, P. (2017) Dynamic factor models with infinite-dimensional factor space: Asymptotic analysis. Journal of Econometrics, 199(1), 74--92.
 #' @references Owens, D., Cho, H. & Barigozzi, M. (2022) fnets: An R Package for Network Estimation and Forecasting via Factor-Adjusted VAR Modelling. arXiv preprint arXiv:2301.11675.
 #' @examples
+#' \dontrun{
 #' set.seed(123)
 #' n <- 500
 #' p <- 50
@@ -30,13 +32,14 @@
 #' x <- common$data + idio$data
 #' out <- fnets(x, q = NULL, var.order = 1, var.method = "lasso",
 #' do.lrpc = FALSE, var.args = list(n.cores = 2))
-#' cpre <- common.predict(out, x, h = 1, r = NULL)
-#' ipre <- idio.predict(out, x, cpre, h = 1)
-#' @export
+#' cpre <- common.predict(out)
+#' ipre <- idio.predict(out, cpre)
+#' }
+#' @keywords internal
 common.predict <-
   function(object,
            x,
-           h = 1,
+           n.ahead = 1,
            fc.restricted = TRUE,
            r = c("ic", "er")) {
     xx <- x - object$mean.x
@@ -48,14 +51,13 @@ common.predict <-
     } else
       r.method <- NULL
 
-    pre <- list(is = 0 * x, fc = matrix(0, nrow = p, ncol = h))
+    pre <- list(is = 0 * x, fc = matrix(0, nrow = p, ncol = n.ahead))
     if(attr(object, "factor") == "unrestricted") {
       if(object$q < 1) {
-        warning(paste0(
+        warning(
           "There should be at least one factor for common component estimation!"
-        ))
-      }
-      if(object$q >= 1) {
+        )
+      } else {
         if(fc.restricted)
           pre <-
             common.restricted.predict(
@@ -65,28 +67,22 @@ common.predict <-
               q = object$q,
               r = r,
               r.method = r.method,
-              h = h
+              n.ahead = n.ahead
             )
-        if(!fc.restricted)
+        else
           pre <-
             common.unrestricted.predict(xx = xx,
                                         cve = object,
-                                        h = h)
+                                        n.ahead = n.ahead)
       }
     }
 
     if(attr(object, "factor") == "restricted") {
       if(object$q < 1) {
-        warning(paste0(
-          "There should be at least one factor for common component estimation!"
-        ))
+        warning("There should be at least one factor for common component estimation!")
       } else if(object$q >= 1) {
       if(!fc.restricted)
-        warning(
-          paste0(
-            "fc.restricted is being set to TRUE, as fnets object is generated with fm.restricted = TRUE"
-          )
-        )
+        warning("fc.restricted is being set to TRUE, as fnets object is generated with fm.restricted = TRUE")
       pre <-
         common.restricted.predict(
           xx = xx,
@@ -95,7 +91,7 @@ common.predict <-
           q = object$q,
           r = object$q,
           r.method = r.method,
-          h = h
+          n.ahead = n.ahead
         )
 
       }
@@ -104,9 +100,6 @@ common.predict <-
   }
 
 #' @title Blockwise VAR estimation under GDFM
-#' @references Barigozzi, M., Cho, H. & Owens, D. (2022) FNETS: Factor-adjusted network estimation and forecasting for high-dimensional time series. arXiv preprint arXiv:2201.06110.
-#' @references Forni, M., Hallin, M., Lippi, M. & Zaffaroni, P. (2017). Dynamic factor models with infinite-dimensional factor space: Asymptotic analysis. Journal of Econometrics, 199(1), 74--92.
-#' @references Owens, D., Cho, H. & Barigozzi, M. (2022) fnets: An R Package for Network Estimation and Forecasting via Factor-Adjusted VAR Modelling. arXiv preprint arXiv:2301.11675.
 #' @keywords internal
 common.irf.estimation <-
   function(xx,
@@ -126,32 +119,23 @@ common.irf.estimation <-
       max.var.order <-
       min(factor.var.order, max(1, ceiling(10 * log(n, 10) / (q + 1) ^ 2)), 10)
 
-    if(q < 1)
-      warning(paste0(
-        "There should be at least one factor for common component estimation!"
-      ))
-
-    if(q >= 1) {
+    if(q < 1){
+      warning("There should be at least one factor for common component estimation!")
+    } else {
       # for each permutation, obtain IRF and u (with cholesky identification), average the output, what FHLZ 2017 do
       irf.array <- array(0, dim = c(p, q, trunc.lags + 2, n.perm))
       u.array <- array(0, dim = c(q, n, n.perm))
 
       for (ii in 1:n.perm) {
-        if(ii == 1)
-          perm.index <- 1:p
-        else
-          perm.index <- sample(p, p)
+        ifelse(ii == 1, perm.index <- 1:p, perm.index <- sample(p, p))
         Gamma_c_perm <- Gamma_c[perm.index, perm.index,]
 
         A <- list()
         z <- xx[perm.index,]
         z[, 1:max.var.order] <- NA
         for (jj in 1:N) {
-          if(jj == N)
-            block <-
-              ((jj - 1) * (q + 1) + 1):p
-          else
-            block <- ((jj - 1) * (q + 1) + 1):(jj * (q + 1))
+          ifelse(jj == N, block <- ((jj - 1) * (q + 1) + 1):p,
+                 block <- ((jj - 1) * (q + 1) + 1):(jj * (q + 1)))
           pblock <- perm.index[block]
           nblock <- length(block)
 
@@ -177,11 +161,7 @@ common.irf.estimation <-
 
         tmp.irf <- irf.array[, , , 1, drop = FALSE] * 0
         for (jj in 1:N) {
-          if(jj == N)
-            block <-
-              ((jj - 1) * (q + 1) + 1):p
-          else
-            block <- ((jj - 1) * (q + 1) + 1):(jj * (q + 1))
+          ifelse(jj == N, block <- ((jj - 1) * (q + 1) + 1):p, block <- ((jj - 1) * (q + 1) + 1):(jj * (q + 1)))
           pblock <- perm.index[block]
           invA <- var.to.vma(A[[jj]], trunc.lags + 1)
           for (ll in 1:(trunc.lags + 2)) {
@@ -219,15 +199,15 @@ common.restricted.predict <-
            r = NULL,
            max.r = NULL,
            r.method = NULL,
-           h = 1) {
+           n.ahead = 1) {
 
     p <- dim(xx)[1]
     n <- dim(xx)[2]
     if(is.null(max.r))
       max.r <- max(q, min(50, round(sqrt(min(n, p)))))
-    if(h >= dim(Gamma_c)[3]) {
-      warning(paste0("At most ", (dim(Gamma_c)[3] - 1) / 2, "-step ahead forecast is available!"))
-      h <- (dim(Gamma_c)[3] - 1) / 2
+    if(n.ahead >= dim(Gamma_c)[3]) {
+      warning("At most ", (dim(Gamma_c)[3] - 1) / 2, "-step ahead forecast is available!")
+      n.ahead <- (dim(Gamma_c)[3] - 1) / 2
     }
 
     if(is.null(r)) {
@@ -236,7 +216,7 @@ common.restricted.predict <-
         r  <- max(q, abc$q.hat[5])
         sv <- abc$sv
       }
-      if(r.method == "er") {
+      else if(r.method == "er") {
         sv <- svd(Gamma_x[, , 1], nu = max.r, nv = 0)
         r <- which.max(sv$d[q:max.r] / sv$d[1 + q:max.r]) + q - 1
       }
@@ -245,11 +225,11 @@ common.restricted.predict <-
 
     is <-
       sv$u[, 1:r, drop = FALSE] %*% t(sv$u[, 1:r, drop = FALSE]) %*% xx
-    if(h >= 1) {
-      fc <- matrix(0, nrow = p, ncol = h)
+    if(n.ahead >= 1) {
+      fc <- matrix(0, nrow = p, ncol = n.ahead)
       proj.x <-
         t(t(sv$u[, 1:r, drop = FALSE]) / sv$d[1:r]) %*% t(sv$u[, 1:r, drop = FALSE]) %*% xx[, n]
-      for (hh in 1:h)
+      for (hh in 1:n.ahead)
         fc[, hh] <- t(Gamma_c[, , hh + 1]) %*% proj.x
     } else {
       fc <- NA
@@ -258,18 +238,18 @@ common.restricted.predict <-
     out <- list(is = is,
                 fc = fc,
                 r = r,
-                h = h)
+                n.ahead = n.ahead)
     return(out)
   }
 
 #' @keywords internal
-common.unrestricted.predict <- function(xx, cve, h = 1) {
+common.unrestricted.predict <- function(xx, cve, n.ahead = 1) {
   p <- dim(xx)[1]
   n <- dim(xx)[2]
   trunc.lags <- dim(cve$loadings)[3]
-  if(h >= trunc.lags + 1) {
-    warning(paste0("At most ", trunc.lags, "-step ahead forecast is available!"))
-    h <- trunc.lags
+  if(n.ahead >= trunc.lags + 1) {
+    warning("At most ", trunc.lags, "-step ahead forecast is available!")
+    n.ahead <- trunc.lags
   }
 
   irf <- cve$loadings
@@ -282,9 +262,9 @@ common.unrestricted.predict <- function(xx, cve, h = 1) {
     is[, (trunc.lags + 1):n] <-
     is[, (trunc.lags + 1):n] + as.matrix(irf[, , ll]) %*% u[, (trunc.lags + 1):n - ll + 1, drop = FALSE]
 
-  if(h >= 1) {
-    fc <- matrix(0, nrow = p, ncol = h)
-    for (hh in 1:h)
+  if(n.ahead >= 1) {
+    fc <- matrix(0, nrow = p, ncol = n.ahead)
+    for (hh in 1:n.ahead)
       for (ll in 1:(trunc.lags + 1 - hh))
         fc[, hh] <-
           fc[, hh] + as.matrix(irf[, , ll + hh]) %*% u[, n - ll + 1, drop = FALSE]
@@ -292,7 +272,7 @@ common.unrestricted.predict <- function(xx, cve, h = 1) {
     fc <- NA
   }
 
-  out <- list(is = is, fc = fc, h = h)
+  out <- list(is = is, fc = fc, n.ahead = n.ahead)
   return(out)
 }
 
@@ -305,13 +285,9 @@ common.yw.est <- function(Gcp, block, var.order) {
   for (ll in 1:var.order) {
     B[, nblock * (ll - 1) + 1:nblock] <- t(Gcp[block, block, 1 + ll])
     for (lll in 1:var.order) {
-      if(ll >= lll) {
-        C[nblock * (ll - 1) + 1:nblock, nblock * (lll - 1) + 1:nblock] <-
-          Gcp[block, block, 1 + ll - lll]
-      } else {
-        C[nblock * (ll - 1) + 1:nblock, nblock * (lll - 1) + 1:nblock] <-
-          t(Gcp[block, block, 1 + lll - ll])
-      }
+        ifelse(ll >= lll,
+               C[nblock * (ll - 1) + 1:nblock, nblock * (lll - 1) + 1:nblock] <-Gcp[block, block, 1 + ll - lll],
+               C[nblock * (ll - 1) + 1:nblock, nblock * (lll - 1) + 1:nblock] <-t(Gcp[block, block, 1 + lll - ll]))
     }
   }
   A <- B %*% solve(C, symmetric = TRUE)
