@@ -2,18 +2,19 @@
 #' @description Performs factor modelling under either restricted (static) or unrestricted (dynamic) factor models
 #' @details See Barigozzi, Cho and Owens (2022) for further details.
 #'
-#' @param x input time series matrix, with each row representing a variable
+#' @param x input time series matrix, with each row representing a variable and each column containing the observations at a given time
 #' @param center whether to de-mean the input \code{x} row-wise
 #' @param fm.restricted whether to estimate a restricted factor model using static PCA
 #' @param q Either a string specifying the factor number selection method when \code{fm.restricted = TRUE}; possible values are:
 #' \itemize{
-#'    \item{\code{"ic"}}{ information criteria of Hallin and Liška (2007) or Bai and Ng (2002), see \link[fnets]{factor.number}}
-#'    \item{\code{"er"}}{ eigenvalue ratio}
+#'    \item{\code{"ic"}}{ information criteria-based methods of Alessi, Barigozzi & Capasso (2010) when \code{fm.restricted = TRUE} or Hallin and Liška (2007) when \code{fm.restricted = FALSE}}
+#'    \item{\code{"er"}}{ eigenvalue ratio of Ahn and Horenstein (2013) when \code{fm.restricted = TRUE} or Avarucci et al. (2022) when \code{fm.restricted = FALSE}}
 #' }
-#' or the number of unrestricted factors.
+#' or the number of unrestricted factors, see \link[fnets]{factor.number}
 #' @param ic.op choice of the information criterion penalty, see \link[fnets]{hl.factor.number} or \link[fnets]{abc.factor.number} for further details
-#' @param kern.bw kernel bandwidth for dynamic PCA; by default, it is set to
-#' \code{4 * floor((dim(x)[2]/log(dim(x)[2]))^(1/3)))}. When \code{fm.restricted = TRUE}, it is used to compute the number of lags for which autocovariance matrices are estimated
+#' @param kern.bw a positive integer specifying the kernel bandwidth for dynamic PCA;
+#' by default, it is set to \code{floor(4 *(dim(x)[2]/log(dim(x)[2]))^(1/3)))}.
+#' When \code{fm.restricted = TRUE}, it is used to compute the number of lags for which autocovariance matrices are estimated
 #' @param common.args a list specifying the tuning parameters required for estimating the impulse response functions and common shocks. It contains:
 #' \itemize{
 #'    \item{\code{factor.var.order}}{ order of the blockwise VAR representation of the common component. If \code{factor.var.order = NULL}, it is selected blockwise by Schwarz criterion}
@@ -29,19 +30,21 @@
 #' a list containing estimators of the impulse response functions (as an array of dimension \code{(p, q, trunc.lags + 2)})}
 #' \item{factors}{ if \code{fm.restricted = TRUE}, factor series; else, common shocks (an array of dimension \code{(q, n)})}
 #' \item{mean.x}{ if \code{center = TRUE}, returns a vector containing row-wise sample means of \code{x}; if \code{center = FALSE}, returns a vector of zeros}
+#' @references Ahn, S. C. & Horenstein, A. R. (2013) Eigenvalue ratio test for the number of factors. Econometrica, 81(3), 1203--1227.
 #' @references Alessi, L., Barigozzi, M.,  & Capasso, M. (2010) Improved penalization for determining the number of factors in approximate factor models. Statistics & Probability Letters, 80(23-24):1806–1813.
-#' @references Bai, J. & Ng, S. (2002) Determining the number of factors in approximate factor models. Econometrica. 70: 191-221.
+#' @references Avarucci, M., Cavicchioli, M., Forni, M., & Zaffaroni, P. (2022) The main business cycle shock(s): Frequency-band estimation of the number of dynamic factors.
 #' @references Barigozzi, M., Cho, H. & Owens, D. (2022) Factor-adjusted network estimation and forecasting for high-dimensional time series. arXiv preprint arXiv:2201.06110.
 #' @references Hallin, M. & Liška, R. (2007) Determining the number of factors in the general dynamic factor model. Journal of the American Statistical Association, 102(478), 603--617.
 #' @references Owens, D., Cho, H. & Barigozzi, M. (2022) fnets: An R Package for Network Estimation and Forecasting via Factor-Adjusted VAR Modelling. arXiv preprint arXiv:2301.11675.
+#' @seealso \link[fnets]{print.fm}, \link[fnets]{predict.fm}
 #' @examples
 #' \donttest{
-#' set.seed(123)
+#' set.seed(1234)
 #' n <- 500
 #' p <- 50
 #' common <- sim.restricted(n, p)
-#' x <- common$data + rnorm(n*p)
-#' out <- fnets.factor.model(x, fm.restricted = TRUE)
+#' x <- common$data + matrix(rnorm(n * p), nrow = p)
+#' out <- fnets.factor.model(x, fm.restricted = FALSE)
 #' }
 #' @export
 fnets.factor.model <-
@@ -57,37 +60,46 @@ fnets.factor.model <-
              trunc.lags = 20,
              n.perm = 10
            )) {
+    x <- as.matrix(x)
     p <- dim(x)[1]
     n <- dim(x)[2]
 
-    if(center)
-      mean.x <- apply(x, 1, mean)
-    else
-      mean.x <- rep(0, p)
+    if(!is.null(ic.op)) ic.op <- max(1, min(as.integer(ic.op), 6))
+
+    ifelse(center, mean.x <- apply(x, 1, mean), mean.x <- rep(0, p))
     xx <- x - mean.x
 
     if(!is.numeric(q)) {
       q.method <- match.arg(q, c("ic", "er"))
       q <- NULL
-    } else
+    } else {
+      q <- posint(q, 0)
       q.method <- NULL
+    }
+
+    args <- as.list(environment())
 
     common.args <- check.list.arg(common.args)
-    if(is.null(kern.bw))
-      kern.bw <- 4 * floor((n / log(n))^(1/3))
+    ifelse(is.null(kern.bw),
+           kern.bw <- 4 * floor((n / log(n))^(1/3)),
+           kern.bw <- max(0, kern.bw))
     mm <- min(max(1, kern.bw), floor(n / 4) - 1)
 
     #if(!is.null(q.method)) q.method <- match.arg(q.method, c("ic", "er"))
     if(fm.restricted) {
-      spca <-
-        static.pca(
-          xx,
-          q = q,
-          q.method = q.method,
-          ic.op = ic.op,
-          mm = mm
-        )
-      q <- spca$q
+      if(isTRUE(q==0)){
+        spca <- list()
+      } else {
+        spca <-
+          static.pca(
+            xx,
+            q = q,
+            q.method = q.method,
+            ic.op = ic.op,
+            mm = mm
+          )
+        q <- spca$q
+      }
       loadings <- spca$lam
       factors <- spca$f
       spec <- NULL
@@ -107,8 +119,10 @@ fnets.factor.model <-
         trunc.lags = common.args$trunc.lags,
         n.perm = common.args$n.perm
       )
-      loadings <- cve$irf.est
-      factors <- cve$u.est
+      if (q >= 1) {
+        loadings <- cve$irf.est
+        factors <-  cve$u.est
+      } else loadings <- factors <- NULL
     }
     out <- list(
       q = q,
@@ -118,15 +132,12 @@ fnets.factor.model <-
       acv = acv,
       mean.x = mean.x
     )
-    if(fm.restricted)
-      attr(out, "factor") <-
-      "restricted"
-    else
-      attr(out, "factor") <- "unrestricted"
-    attr(out, "class") <- "fm" #"fnets"
-
+    ifelse(fm.restricted,attr(out, "factor") <-"restricted",
+           attr(out, "factor") <- "unrestricted")
+    attr(out, "class") <- "fm"
+    attr(out, "args") <- args
     return(out)
-  }
+}
 
 #' @title Dynamic PCA
 #' @description Performs principal components analysis in frequency domain for identifying common and idiosyncratic components.
@@ -134,7 +145,7 @@ fnets.factor.model <-
 #' @param q number of factors. If \code{q = NULL}, the factor number is estimated by an information criterion-based approach of Hallin and Liška (2007)
 #' @param q.method A string specifying the factor number selection method; possible values are:
 #' \itemize{
-#'    \item{\code{"ic"}}{ information criteria-based methods of Alessi, Barigozzi & Capasso (2010) when \code{fm.restricted = TRUE} or Hallin and Liška (2007) when \code{fm.restricted = FALSE} modifying Bai and Ng (2002)}
+#'    \item{\code{"ic"}}{ information criteria-based methods of Alessi, Barigozzi & Capasso (2010) when \code{fm.restricted = TRUE} or Hallin and Liška (2007) when \code{fm.restricted = FALSE}}
 #'    \item{\code{"er"}}{ eigenvalue ratio of Ahn and Horenstein (2013)}
 #' }
 #' @param ic.op choice of the information criterion penalty. Currently the three options from Hallin and Liška (2007) (\code{ic.op = 1, 2} or \code{3}) and
@@ -198,6 +209,8 @@ dyn.pca <-
         for (ii in 1:(mm + 1)){
           eigs <- eigs + sv[[ii]]$d[0:q + 1]
         }
+        q.method.out <- eigs[1:q] / eigs[1 + 1:q]
+        q <- which.max(q.method.out)
       } else if(q.method == "ic") {
         q.max <- min(50, floor(sqrt(min(n - 1, p))))
         q.method.out <-
@@ -206,7 +219,6 @@ dyn.pca <-
       }
     }
 
-
     Gamma_c <- Gamma_i <- Sigma_c <- Sigma_i <- Sigma_x * 0
     if(q >= 1) {
       for (ii in 1:(mm + 1)) {
@@ -237,11 +249,11 @@ dyn.pca <-
     out <-
       list(
         q = q,
-        # q.method.out = q.method.out,
         spec = spec,
         acv = acv,
         kern.bw = kern.bw
       )
+    if(q.method == "er" & flag) out$q.method.out <- q.method.out
     return(out)
   }
 
@@ -269,6 +281,7 @@ static.pca <-
     covx <- xx %*% t(xx) / n
     eig <- eigen(covx, symmetric = TRUE)
 
+    q.method.out <- 0
     if(is.null(q)) {
       if(q.method == "er") {
         q.method.out <- eig$values[1:q.max] / eig$values[1 + 1:q.max]
@@ -281,7 +294,6 @@ static.pca <-
             xx,
             covx = covx,
             q.max = q.max,
-            do.plot = FALSE,
             center = FALSE
           )
         q <- q.method.out$q.hat[ic.op]
@@ -323,43 +335,76 @@ static.pca <-
 
 #' @title Forecasting for factor models
 #' @method predict fm
-#' @description Produces forecasts of the data for a given forecasting horizon by
+#' @description Produces forecasts of the data input to \code{object} for a given forecasting horizon by
 #'estimating the best linear predictors of the common component
 #' @param object \code{fm} object
-#' @param x input time series matrix, with each row representing a variable
-#' @param h forecasting horizon
-#' @param fc.restricted whether to forecast using a restricted or unrestricted, blockwise VAR representation of the common component
-#' @param r number of restricted factors, or a string specifying the factor number selection method when \code{fc.restricted = TRUE};
-#'  possible values are:
+#' @param n.ahead forecasting horizon
+#' @param fc.restricted if \code{fc.restricted = TRUE}, the forecast is generated under a restricted factor model
+#' @param r number of static factors, or a string specifying the factor number selection method when \code{fc.restricted = TRUE};
+#'possible values are:
 #' \itemize{
 #'    \item{\code{"ic"}}{ information criteria of Alessi, Barigozzi & Capasso (2010)}
-#'    \item{\code{"er"}}{ eigenvalue ratio}
+#'    \item{\code{"er"}}{ eigenvalue ratio of Ahn & Horenstein (2013) }
 #' }
 #' @param ... not used
 #' @return a list containing
 #' \item{is}{ in-sample predictions}
 #' \item{forecast}{ forecasts for the given forecasting horizon}
 #' \item{r}{ factor number}
-#' @references Ahn, S. C. & Horenstein, A. R. (2013) Eigenvalue ratio test for the number of factors. Econometrica, 81(3), 1203--1227.
-#' @references Alessi, L., Barigozzi, M.,  & Capasso, M. (2010) Improved penalization for determining the number of factors in approximate factor models. Statistics & Probability Letters, 80(23-24):1806–1813.
-#' @references Barigozzi, M., Cho, H. & Owens, D. (2022) Factor-adjusted network estimation and forecasting for high-dimensional time series. arXiv preprint arXiv:2201.06110.
-#' @references Owens, D., Cho, H. & Barigozzi, M. (2022) fnets: An R Package for Network Estimation and Forecasting via Factor-Adjusted VAR Modelling. arXiv preprint arXiv:2301.11675.
-#' @seealso \link[fnets]{fnets.factor.model}, \link[fnets]{common.predict}
+#' @seealso \link[fnets]{fnets.factor.model}
+#' @examples
+#' set.seed(123)
+#' n <- 500
+#' p <- 50
+#' common <- sim.restricted(n, p)
+#' x <- common$data + rnorm(n*p)
+#' out <- fnets.factor.model(x, fm.restricted = TRUE)
+#' pre <- predict(out)
 #' @export
 predict.fm <-
   function(object,
-           x,
-           h = 1,
+           n.ahead = 1,
            fc.restricted = TRUE,
            r = c("ic", "er"),
            ...) {
+    x <- attr(object, "args")$x
+    n.ahead <- posint(n.ahead)
     out <-
       common.predict(
         object = object,
         x = x,
-        h = h,
+        n.ahead = n.ahead,
         fc.restricted = fc.restricted,
         r = r
       )
     return(out)
   }
+
+#' @title Print factor model
+#' @method print fm
+#' @description Prints a summary of a \code{fm} object
+#' @param x \code{fm} object
+#' @param ... not used
+#' @return NULL, printed to console
+#' @seealso \link[fnets]{fnets.factor.model}
+#' @examples
+#' set.seed(123)
+#' n <- 500
+#' p <- 50
+#' common <- sim.restricted(n, p)
+#' idio <- sim.var(n, p)
+#' x <- common$data + idio$data
+#' out <- fnets.factor.model(x, q = "ic")
+#' print(out)
+#' @export
+print.fm <- function(x,
+                        ...){
+  args <- attr(x, "args")
+  cat(paste("Factor model with \n"))
+  cat(paste("n: ", args$n, ", p: ", args$p,  "\n", sep = ""))
+  cat(paste("Factor model: ", attr(x, "factor"), "\n", sep = ""))
+  cat(paste("Number of factors: ", x$q, "\n", sep = ""))
+  cat(paste("Factor number selection method: ", ifelse(is.null(args$q.method), "NA", args$q.method), "\n", sep = ""))
+  if(!is.null(args$q.method)) if(args$q.method == "ic")
+    cat(paste("Information criterion: ", ifelse(is.null(args$ic.op), "IC5", paste("IC", args$ic.op, sep = "")), "\n", sep = ""))
+}
